
** -----------------------------------------------------------------------------
** Estimate the hazard rate given local bandwidths
**
capture program drop hazard
program define hazard

	capture drop K
	capture drop z
	qui gen K=0
	qui gen z=0

	forvalues x = $lb/$ub {

		local b=${b`x'}

		if `x'<=$lb+`b' {               // @ lower bound 
			local q = (`x'-$lb)/`b'
			qui replace z = (`x'-dur)/`b'
		}

		else if `x'>=$ub-`b' {          // @ upper bound 
			local q = ($ub-`x')/`b'
			qui replace z = (dur-`x')/`b'
		}

		else {                          // @ interior points
			local q = 1
			qui replace z = (`x'-dur)/`b'
		}

		qui replace K = (abs(z)<=1 & dur>=$lb & dur<=$ub) * ///
		                12 * (z+1) * [z*(1-2*`q') + (3*`q'^2-2*`q'+1)/2] / (Ni*(1+`q')^4)
		qui sum K if dur>=$lb & dur<=$ub
		global h`x' = r(N)*r(mean)/`b'

	}

	qui drop K z

end




** -----------------------------------------------------------------------------
** Estimate hazard function non-parametrically, allowing for right censoring
** Needed to estimate of anticipation effects free from contamination bias from 
**    controls crossing April 1st, 1998
**
capture program drop hazard_rc
program define hazard_rc

	capture drop K
	capture drop z
	qui gen K = 0
	qui gen z = 0

	forvalues x = $lb/$ub {

		local b=${b`x'}

		if `x'<=$lb+`b' {                    // @ lower bound 
			local q = (`x'-$lb)/`b'
			qui replace z = (`x'-durrc)/`b'
		}

		else if `x'>=$ub-`b' {               // @ upper bound 
			local q = ($ub-`x')/`b'
			qui replace z = (durrc-`x')/`b'
		}

		else {                               // @ interior points
			local q = 1
			qui replace z = (`x'-durrc)/`b'
		}

		qui replace K = (abs(z)<=1 & durrc>=$lb & durrc<=$ub) * (1-rc) * 12 * (z+1) * [z*(1-2*`q') + (3*`q'^2-2*`q'+1)/2] / (Ni*(1+`q')^4)
		qui sum K if durrc>=$lb & durrc<=$ub
		global h`x' = r(N)*r(mean)/`b'

	}

	qui drop K z

end


** -----------------------------------------------------------------------------
** Estimate the hazard rate between durations lb and ub Muller and Wang local 
**     estimator with adaptive kernel and bandwidth
**
cap program drop adaptiveh
program define adaptiveh

** Inputs:
**   lb and ub (globals): define estimation interval (in duration)
**   b0 (global): initial guess for the bandwidth
**   dur (data variable): contains information on duration of the spell
** Outputs: variables marked "output"
**   h0: hazard rates using constant bandwidth
**   ob: optimal local bandwidth
**   obsmooth: smoothed optimal local bandwidth
**   hob: MW adaptive hazard


	** Construct (n-i+1) denominator: count survivors

		qui cap drop Ni
		qui gen Ni=.
		qui sum dur
		local maxdist=r(max)
		forvalues i=1/`maxdist' {
			qui count if dur>=`i'
			qui replace Ni = r(N) if dur == `i'
		}

	
	** Estimate hazard rates using b0

		forvalues i=$lb/$ub {
			global b`i'=$b0
		}
		hazard
		cap drop h0
		gen h0=.
		forvalues x=$lb/$ub {
			qui replace h0=${h`x'} if _n==`x'
		}
		label var h0 "output: hazard w/ ctt bandwidth"

		
	** Calculate empirical survival function

		qui cap drop d
		qui gen d = .
		qui cap drop Ln
		qui gen Ln = .
		forvalues x=$lb/$ub {
			qui count if dur>=`x'
			qui replace Ln = r(N)/_N if _n==`x'
		}
		qui drop d
		label var Ln "output: empirical survival fc"


	** Calculate optimal local bandwidths

		* grid in bandwidth
		global Nb=60
		forvalues j=1/$Nb  {
			global b`j' = $b0/6 + (`j'-1)*(6*$b0-$b0/6)/($Nb-1)
		}

		* nr pts in duration (around $x below)
		global Nd=40

		* calculate optimal local bandwidth by minimising local MSE
		qui cap drop ob
		qui gen ob=.
		forvalues i = $lb/$ub {
			global x=`i'
			localMSE
			qui sum MSEx
			forvalues j=$Nb(-1)1 {
				if MSEx[`j']==r(min) {
					qui replace ob   = ${b`j'} if _n==`i'
				}
			}
			di "optimal bdwidth at duration ", `i', " is ", ob[`i']
		}
		label var ob "output: optimal local bandwidth"	
		qui drop vx bx MSEx

		* Smooth optimal local bandwidths using LLR w/ global bandwidth
		smoothb
		label var obsmooth "output: smoothed optimal local bandwidth"


	** Calculate hazard rates using optimal local bandwidth

		forvalues i=$lb/$ub {
			global b`i'=obsmooth[`i']
		}
		hazard
		qui capture drop hob
		qui gen hob=.
		forvalues i=$lb/$ub {
			qui replace hob=${h`i'} if _n==`i'
		}
		label var hob "output: hazard w/ adaptive bandwidth"

		qui drop Ni
	
end




** -----------------------------------------------------------------------------
** Estimate the hazard rate at duration d using Muller and Wang local estimator 
**    with adaptive kernel and bandwidth
** Suited to estimate hazards towards a boundary
** Quicker version than that in adaptiveh, designed for bootstrap
**
cap program drop adaptiveh_bs
program define adaptiveh_bs

** Inputs:                                                                              **
**		lb and ub (both globals): define interval (in duration) of estimation           **
**		mini and maxi (both globals): define interval in which to compute opt bdw       **
**		pt (global): pt of interst, cutoff pt                                           **
**		b0 (global): initial guess for the bandwidth                                    **
**		dur (data variable): contains information on duration of the spell              **
** Outputs:                                                                             **
**		h0 (data variable): hazard rates with initial cte bdw                           **
**		ob_bs (data variable): optimal bdw in the interval mini-maxi                    **
**		obpt (global): smoothed optimal bdw at pt of interest, pt                       **
**		h`x' (globals): hazard rates using bdw=obpt over lb-ub                          ** 


	** Create the (n-i+1) denominator
		qui cap drop Ni
		qui gen Ni=.
		qui sum dur
		local maxdist=r(max)
		forvalues i=1/`maxdist' {
			qui count if dur>=`i'
			qui replace Ni = r(N) if dur == `i'
		}


	** Estimate hazard rates using b0
		forvalues i=$lb/$ub {
			global b`i'=$b0
		}
		qui hazard
		qui capture drop h0
		qui gen h0=.
		forvalues x=$lb/$ub {
			qui replace h0=${h`x'} if _n==`x'
		}
		qui label var h0 "output: hazard using initial cte bdw"


	** Create the empirical survival function
		qui cap drop d
		qui gen d = .
		qui cap drop Ln
		qui gen Ln = .
		forvalues x=$lb/$ub {
			qui replace d = (dur<=`x')
			qui su d
			qui replace Ln = 1 - _N*r(mean)/(_N+1) if _n==`x'
		}
		qui drop d


	** Compute optimal local bandwidths
		global Nb=60
		forvalues j=1/$Nb  {
			global b`j' = $b0/6 + (`j'-1)*(6*$b0-$b0/6)/($Nb-1)
		}
		
		global Nd=40

		qui cap drop ob_bs
		qui gen ob_bs=.
		forvalues i = $mini/$maxi {
			global x=`i'
			qui localMSE
			qui sum MSEx
			forvalues j=$Nb(-1)1 {
				if MSEx[`j']==r(min) {
					qui replace ob_bs = ${b`j'} if _n==`i'
				}
			}
		}
		qui label var ob_bs "output: optimal bdw close to 'ptԢ
		global ob = ob_bs[$pt]

		qui drop vx bx MSEx
	

	** Smooth optimal local bandwidths using LLR w/ global bandwidth
		qui smoothb_bs


	** Obtain hazard rates using optimal bandwidth
		forvalues i=$lb/$ub {
			global b`i'=$obpt
		}
		qui hazard
		qui drop Ni Ln

end




** -----------------------------------------------------------------------------
** Calculates hazard rates using Muller and Wang local estimator with adaptive 
**    bandwidth. allowing for the possibility of right censoring (other routines 
**    exclude it)
** Particularly suited to estimate hazards towards a boundary
** See adaptiveh_bs for further details
**
cap program drop adaptiveh_rc_bs
program define adaptiveh_rc_bs

	** Create the (n-i+1) denominator
		qui cap drop Ni
		qui gen Ni=.
		qui sum durrc
		local maxdist=r(max)
		forvalues i=1/`maxdist' {
			qui count if durrc>=`i'
			qui replace Ni = r(N) if durrc == `i'
		}


	** Estimate hazard rates using b0
		forvalues i=$lb/$ub {
			global b`i'=$b0
		}
		qui hazard_rc
		qui capture drop h0
		qui gen h0=.
		forvalues x=$lb/$ub {
			qui replace h0=${h`x'} if _n==`x'
		}
		qui label var h0 "output: hazard using initial ctt bdw"


	** Create the empirical survival function
		qui cap drop d
		qui gen d = .
		qui cap drop Ln
		qui gen Ln = .
		scalar N = _N
		scalar left = 0
		forvalues x=1/$ub {
			qui replace d = (durrc>=`x')
			qui sum d
			qui replace Ln = N*r(mean)/N if _n==`x'
			qui count if durrc==`x' & rc==1
			scalar N = N-r(N)
		}
		qui drop d


	** Compute optimal local bandwidths
		global Nb=60
		forvalues j=1/$Nb  {
			global b`j' = $b0/6 + (`j'-1)*(6*$b0-$b0/6)/($Nb-1)
		}
		
		global Nd=40

		qui cap drop ob_bs
		qui gen ob_bs=.
		forvalues i = $mini/$maxi {
			global x=`i'
			qui localMSE
			qui sum MSEx 
			forvalues j=$Nb(-1)1 {
				if MSEx[`j']==r(min) {
					qui replace ob_bs = ${b`j'} if _n==`i'
				}
			}
		}
		qui label var ob_bs "output: optimal bdw close to 'ptԢ
		global ob = ob_bs[$pt]

		qui drop vx bx MSEx


	** Smooth optimal local bandwidths using local linear regression w/ global bandwidth
		qui smoothb_bs


	** Obtain hazard rates using optimal bandwidth
		forvalues i=$lb/$ub {
			global b`i'=$obpt
		}
		qui hazard_rc
		di $obpt, ${h$pt}

		if (abs(189-$pt)<$obpt & $censor189==1) {
			global obpt=abs(189-$pt)
			forvalues i=$lb/$ub {
				global b`i'=$obpt
			}
			qui hazard_rc
			di $obpt, ${h$pt}
		}

		qui drop Ni Ln

end




** -----------------------------------------------------------------------------
** Estimate optimal local MSE
**
capture program drop localMSE
program define localMSE

	cap qui drop vx
	cap qui drop bx
	cap qui drop MSEx
	qui gen vx=.
	qui gen bx=.
	qui gen MSEx=.

	forvalues j = 1/$Nb  {              // for each bandwidth in the grid

		local b=${b`j'}

		if $x<$lb+`b' {                 // compute boundaries of estimation interval
			local q = ($x-$lb)/`b'
			local maxy = `q'
			local miny = -1
		}
		else if $x>=$ub-`b' {
			local q = ($ub-$x)/`b'
			local maxy = 1 
			local miny = -`q'
		}
		else {
			local q = 1
			local maxy = 1
			local miny = -1
		}

		local vx = 0				    // initialise values to compute var and bias
		local bx = 0

		forvalues l = 0/$Nd {			// for each point in the grid for y (standardised x)

			local y = `miny'+`l'*(`maxy'-`miny')/$Nd
			local z = $x-`b'*`y'

			if $x>=$ub-`b' {
				local Ky = 12 * (-`y'+1) * [-`y'*(1-2*`q') + (3*`q'^2-2*`q'+1)/2] / ((1+`q')^4)
			}
			else {
				local Ky = 12 * (`y'+1) * [`y'*(1-2*`q') + (3*`q'^2-2*`q'+1)/2] / ((1+`q')^4)
			}
			
			local zlb = max(int(`z'),$lb)
			local zub = min(`zlb'+1,$ub)
			local w   = `zub' - `z'
			local hy  = `w'*h0[`zlb']+(1-`w')*h0[`zub']
			local Ly  = `w'*Ln[`zlb']+(1-`w')*Ln[`zub']

			if (`l'==0 | `l'==$Nd) {
				local vx = `vx'+`Ky'^2*`hy'/(2*`Ly')
				local bx = `bx'+`Ky'*`hy'/2
			}
			else {
				local vx = `vx'+`Ky'^2*`hy'/`Ly'
				local bx = `bx'+`Ky'*`hy'
			}
		}

		qui replace vx = `vx'*(`maxy'-`miny')/($Nd*_N*`b') if _n==`j'
		qui replace bx = `bx'*(`maxy'-`miny')/$Nd - h0[$x] if _n==`j'

	}

	qui replace MSEx = vx + (bx)^2

end



** -----------------------------------------------------------------------------
** Smooth optimal local bandwidth for routine adaptiveh
**
capture program drop smoothb
program define smoothb

	capture drop K
	capture drop Kb
	capture drop z
	capture drop obsmooth
	qui gen K=0
	qui gen z=0
	qui gen Kb=0
	qui gen obsmooth=0

	local b=$b0

	capture drop obi
	qui gen obi=.
	local N=_N
	forvalues x = $lb/$ub {
		qui replace obi = ob[`x'] if dur==`x'
	}

	forvalues x = $lb/$ub {

		if `x'<=$lb+`b' {                   // @ lower bound
			local q = (`x'-$lb)/`b'
			qui replace z = (`x'-dur)/`b'
		}

		else if `x'>=$ub-`b' {              // @ upper bound 
			local q = ($ub-`x')/`b'
			qui replace z = (dur-`x')/`b'
		}

		else {                              // @ interior point
			local q = 1
			qui replace z = (`x'-dur)/`b'
		}

		qui replace K = (abs(z)<=1 & dur>=$lb & dur<=$ub) * 12 * (z+1) * [z*(1-2*`q') + (3*`q'^2-2*`q'+1)/2] / (Ni*(1+`q')^4)
		qui replace Kb = K*obi
		qui sum K if obi~=.
		local sumK=r(mean)
		qui sum Kb if obi~=.
		local sumKb=r(mean)
		qui replace obsmooth = `sumKb'/`sumK' if _n==`x'

	}

	qui drop K Kb z obi

end




** -----------------------------------------------------------------------------
** Smooth optimal local bandwidth for routine adaptiveh_bs
**
capture program drop smoothb_bs
program define smoothb_bs

	capture drop K
	capture drop Kb
	capture drop z
	qui gen K=0
	qui gen z=0
	qui gen Kb=0

	local b=$b0

	capture drop obi
	qui gen obi=.
	local N=_N
	forvalues x = $lb/$ub {
		qui replace obi = ob_bs[`x'] if dur==`x'
	}

	local q = 0                         // bootstrap st error at cutoff point

	if $pt==$lb {                       // st distance if @ lower bound
		qui replace z = ($pt-dur)/`b'
	}
	else if $pt==$ub {				    // st distance if @ upper bound
		qui replace z = (dur-$pt)/`b'
	}

	qui replace K = (abs(z)<=1 & dur>=$lb & dur<=$ub) * 12 * (z+1) * [z*(1-2*`q') + (3*`q'^2-2*`q'+1)/2] / (Ni*(1+`q')^4)
	qui replace Kb = K*obi
	qui sum K if obi~=.
	local sumK=r(mean)
	qui sum Kb if obi~=.
	local sumKb=r(mean)
	global obpt = `sumKb'/`sumK'
	
	qui drop K Kb z obi

end



